/*
 * File: imageproc.c 
 * This is the starting point for the tif image processing assignment
 *
 * add your name and a comment with a high-level description of the
 * contents of this file
 *
 * (Richard Wicentowski and Tia Newhall)
 * (2004)
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <tiffio.h>
#include<tcl.h>
#include<tk.h>
#include "tiff.h"

/* DO NOT ALTER THESE NEXT LINES */
#define WHITE 255
#define BLACK 0
/* DO NOT ALTER THESE PREVIOUS LINES */

/* there are two constants defined in tiff.h that you will likely need
 * to use in your code:  
 * 	ROWS for the number of rows in the image and
 * 	COLS for the number of columns in the image
 */

/* the image processing options */
#define NEG		1
#define VER     	2 
#define FLIP    	3
#define SWITCH  	4
#define DARKEN  	5
#define LIGHTEN 	6
#define POLARIZE	7
#define SCROLLV		8
#define SCROLLH		9
#define ZOOM		10
#define BLUR		11
#define ROTATE		12
#define REVERT          13
#define SORT            14
#define EX1             15
#define EX2             16
#define EX3             17
#define EX4             18
#define EX5             19
#define QUIT            20



/** function prototypes: 
 * These are given to you as a starting point guide for implementing these
 * functions; you may change the parameters to these if you'd like 
 * (within reason that is...you must pass the array(s) as parameters). 
 */
void copy_image(unsigned char image[][COLS], unsigned char copy[][COLS]);
void make_negative(unsigned char image[][COLS]);
void flip_horizontal(unsigned char image[][COLS]);
void flip_vertical(unsigned char image[][COLS]);  
void flip_corners(unsigned char image[][COLS]); 
void scroll_vertical(unsigned char image[][COLS], int n); 
void scroll_horizontal(unsigned char image[][COLS], int n); 
int scroll(); 
void blur(unsigned char image[][COLS]); 
void rotate90(unsigned char image[][COLS]);
void zoom_topleft(unsigned char image[][COLS]); 
void zoom_topright(unsigned char image[][COLS]) ; 
void zoom_bottomleft(unsigned char image[][COLS]) ; 
void zoom_bottomright(unsigned char image[][COLS]) ; 
void zoom_center(unsigned char image[][COLS]) ; 
void zoom(unsigned char image[][COLS]) ; 
void lighten(unsigned char image[][COLS]) ; 
void darken(unsigned char image[][COLS]) ; 
void polarize(unsigned char image[][COLS]) ; 
void sort_rows(unsigned char image[][COLS]);
void print_menu();
void perform_action(int choice, unsigned char image[][COLS], 
		unsigned char copy[][COLS]);

/********************* DON'T CHANGE THESE FUNCTIONS *********************/
int InitGUI(Tcl_Interp **interp, char *argv[], char *infile,
		unsigned char image[][COLS]); 
int Redisplay(unsigned char image[][COLS], Tcl_Interp *interp);
int Tk_AppInit(Tcl_Interp *interp);
/************************************************************************/

int main(int argc, char *argv[]) { 
	unsigned char image[ROWS][COLS]; 
	unsigned char copy[ROWS][COLS]; 
	char  infile[100]; 
	Tcl_Interp *interp;
	int choice = 0;

	/****** DO NOT ALTER THESE LINES ******/
  	printf ("Enter the name of a %dx%d greyscale TIFF image: ", ROWS,COLS);
	scanf("%s", infile);
        if(InitGUI(&interp, argv, infile, image)) {
		printf("ERROR: cannot open file: %s\n", infile); 
		exit(1); 
	}
	/**************************************/

	copy_image(image, copy);  /* initialize copy to match image */

	while(choice != QUIT) { 
		print_menu();
		printf ("Your choice: \n"); 
		scanf("%d", &choice);
		perform_action(choice, image, copy); 
		
		Redisplay(image, interp); /* DO NOT ALTER THIS LINE */ 
	} 
	printf("Good bye\n\n");
	return(0);
}

/************************************************************************/
/* Function: print_menu: 
 *  prints menu of options to screen
 */
void print_menu() { 
	printf ("What do you want to do to your image?\n"); 
	printf ("%d. Make a negative\n",NEG); 
	printf ("%d. Flip it vertically\n",VER); 
	printf ("%d. Flip it horizontally\n",FLIP); 
	printf ("%d. Switch the top left and bottom right corners\n", SWITCH); 
	printf ("%d. Darken\n", DARKEN); 
	printf ("%d. Lighten\n", LIGHTEN); 
	printf ("%d. Polarize\n", POLARIZE); 
	printf ("%d. Scroll the image vertically\n",SCROLLV); 
	printf ("%d. Scroll the image horizontally\n",SCROLLH); 
	printf ("%d. Zoom in\n", ZOOM); 
	printf ("%d. Blur the image\n", BLUR); 
	printf ("%d. Rotate 90 degrees\n", ROTATE); 
	printf ("%d. Revert to the original\n", REVERT); 
	printf ("%d. Sort Rows\n", SORT);
	printf ("\nExtra Credit Special Effects\n"); 
	printf ("----------------------------\n"); 
	printf ("%d. \n", EX1); 
	printf ("%d. \n", EX2); 
	printf ("%d. \n", EX3); 
	printf ("%d. \n", EX4); 
	printf ("%d. \n", EX5); 
	printf ("\n%d. Quit\n\n", QUIT);
}
/************************************************************************/
/* Function: perform_action: 
 *  executes the option chosen by the user 
 *
 *  choice: which choice the user chose
 *  image: the image
 *  copy: a copy of the image, neeed by some options
 *  returns: no return value
 */
void perform_action(int choice, unsigned char image[][COLS], 
		unsigned char copy[][COLS]) 
{ 
	switch (choice) { 
		case NEG: make_negative(image); break; 
		case VER: flip_vertical(image); break; 
		case FLIP: flip_horizontal(image); break; 
		case SWITCH: flip_corners(image); break; 
		case DARKEN: darken(image); break; 
		case LIGHTEN: lighten(image); break; 
		case POLARIZE: polarize(image); break; 
		case SCROLLV: scroll_vertical(image, scroll()); break; 
		case SCROLLH: scroll_horizontal(image, scroll()); break; 
		case ZOOM: zoom(image); break; 
		case BLUR: blur(image); break; 
		case ROTATE: rotate90(image); break; 
		case REVERT: copy_image(copy, image); break; 
		case SORT: sort_rows(image); break;
		case EX1: break; 
		case EX2: break; 
		case EX3: break; 
		case EX4: break; 
		case EX5: break; 
		case QUIT: break; 
		default: printf ("Invalid choice: Try again\n"); 
	  }
}
/************************************************************************/
/* Function: copy_image: 
 * Takes the original picture, stored in the array image, and makes a copy 
 * of it into the array called copy. 
 * 	image: the original image
 * 	copy: the copy
 * 	returns: no return value
 */ 
void copy_image(unsigned char image[][COLS], unsigned char copy[][COLS]) { 
	printf("inside copy_image\n");
	return;
}
/************************************************************************/
/* Function: make_negative: 
 * This function turns your image into a negative of your image. 
 * 	image: the image
 * 	returns: no return value
 */ 
void make_negative(unsigned char image[][COLS]) { 
	printf("inside make_negative\n");
	return;
}
/************************************************************************/
/* Function: flip_horizontal: 
 * This function flips the image horizontally to form a mirror image. 
 * 	image: the image
 * 	returns: no return value
 */
void flip_horizontal(unsigned char image[][COLS]) {
	printf("inside flip_hor\n");
	return;
}
/************************************************************************/
/* Function: flip_vertical: 
 * This function turns your image upside down.
 * 	image: the image
 * 	returns: no return value
 */
void flip_vertical(unsigned char image[][COLS]) { 
	printf("inside flip_ver\n");
	return;
}
/************************************************************************/
/* Function: flip_corners: 
 * This function switches the top left and bottom right corners and,
 * if you want, also flips them around. 
 * 	image: the image
 * 	returns: no return value
 */
void flip_corners(unsigned char image[][COLS]) { 
	printf("inside flip_corners\n");
	return;
}
/************************************************************************/
/* Function: scroll_vertical: 
 *  Moves the image UP n pixels if n is negative.
 *  Moves the image DOWN n pixels if n is positive.
 *  Does nothing if n is 0.
 *  
 *  Use mod (%) to make sure that n is not larger than the size of
 *  the image in the UP-DOWN direction.
 *  
 *  HINT: You may want to make a copy of the image to help you do this.
 *
 * 	image: the image
 * 	n: the amount to move the up or down 
 * 	returns: no return value
 */
void scroll_vertical(unsigned char image[][COLS], int n) { 
	printf("inside scroll_vert\n");
	return;
}
/************************************************************************/
/* Function: scroll_horizontal: 
 *  Moves the image to the LEFT n pixels if n is negative.
 *  Moves the image to the RIGHT n pixels if n is positive.
 *  Does nothing if n is 0.
 *  
 *  Use mod (%) to make sure that n is not larger than the size of
 *  the image in the LEFT-RIGHT direction.
 *  
 *  HINT: You may want to make a copy of the image to help you do this.
 * 	image: the image
 * 	n: the amount to move the up or down 
 */
void scroll_horizontal(unsigned char image[][COLS], int n) { 
	printf("inside scroll_hor\n");
	return;
}
/************************************************************************/
/* Function: scroll: 
 * A function which prompts the user to enter how much they want to
 * scroll.  This value is used when calling scroll_horizontal() and
 * scroll_vertical() from the switch statement in main(). 
 *
 * returns: the distance to scroll
 */
int scroll() { 
	int distance = 0;

	printf("inside scroll\n");
	return distance; 
}
/************************************************************************/
/* Function: blur: 
 *  This function will blur the image.  To blur an image, you
 *  substitute a pixel with the average of all the pixels around it.
 *  
 *  To do this properly, you will need to make a copy of the image.
 *  Then, to create the blurred image, use the following algorithm:
 *  
 *  So, if the copy of your image had a section like this:
 *  
 *  +-----------------+
 *  | 116 | 119 | 65  |
 *  +-----------------+
 *  | 85  | 14  | 31  |   
 *  +-----------------+
 *  | 177 |  5  | 55  |
 *  +-----------------+
 *  
 *  In your original image, you would write only the center pixel with
 *  the average of all of the pixels in the 3x3 square:
 *  
 *  +-----------------+
 *  |     |     |     |
 *  +-----------------+
 *  |     | 74  |     | 
 *  +-----------------+
 *  |     |     |     |
 *  +-----------------+
 *  
 *  You DO NOT change the other 8 pixels in your original image, only
 *  the center pixel.
 *
 *   Be careful at the edges of the image: not all pixels have 8 neighboring
 *   pixels!
 *
 *   image: the image
 *   returns: no return value
 */
void blur(unsigned char image[][COLS]) { 
	printf("inside blur\n");
	return;
}
/************************************************************************/
/* Function: rotate90: 
 *  This function rotates the image 90 degrees (you can choose which
 *  direction to rotate).  
 *
 *  image: the image to rotate
 *  returns: no return value
 */
void rotate90(unsigned char image[][COLS]) { 
	printf("inside rotate90\n");
	return;
}
/************************************************************************/
/* Function: zoom_topleft: 
 *  This functions zooms in to the top left 1/4 of the image 
 *
 *  image: the image to zoom
 *  returns: no return value
 */
void zoom_topleft(unsigned char image[][COLS]) { 
	printf("inside zoomtl\n");
	return;
}
/************************************************************************/
/* Function: zoom_topright: 
 * This functions zooms in to the top right 1/4 of the image 
 *
 *  image: the image to zoom
 *  returns: no return value
 */
void zoom_topright(unsigned char image[][COLS]) { 
	printf("inside zoomtr\n");
	return;
}
/************************************************************************/
/* Function: zoom_bottomleft: 
 * This functions zooms in to the bottom left 1/4 of the image 
 *
 *  image: the image to zoom
 *  returns: no return value
 */
void zoom_bottomleft(unsigned char image[][COLS]) { 
	printf("inside zoombl\n");
	return;
}
/************************************************************************/
/* Function: zoom_bottomright: 
 * This functions zooms in to the bottom right 1/4 of the image 
 *
 *  image: the image to zoom
 *  returns: no return value
 */
void zoom_bottomright(unsigned char image[][COLS]) { 
	printf("inside zoombr\n");
	return;
}
/************************************************************************/
/* Function: zoom_center: 
 * This functions zooms in to the center 1/4 of the image 
 *  image: the image to zoom
 *  returns: no return value
 */
void zoom_center(unsigned char image[][COLS]) { 
	printf("inside zoomc\n");
	return;
}
/************************************************************************/
/* Function: zoom: 
 * This function asks the user to choose where they want to zoom,
 *  then calls the function that zooms in the proper area. 
 *  image: the image to zoom
 *  returns: no return value
 */
void zoom(unsigned char image[][COLS]) { 
	printf("inside zoom\n");
	return;
}
/************************************************************************/
/* Function: darken: 
 * This function should darken an image such that values are
 * more BLACK than they were before. 
 *  image: the image to darken
 *  returns: no return value
 */
void darken(unsigned char image[][COLS]) { 
	printf("inside darken\n");
	return;
}
/************************************************************************/
/* Function: lighten: 
 * This function should lighten an image such that values are
 *  more WHITE than they were before. 
 *  image: the image to lighten
 *  returns: no return value
 */
void lighten(unsigned char image[][COLS]) { 
	printf("inside lighten\n");
	return;
}
/************************************************************************/
/* Function: polarize: 
 * This function should polarize an image such that values below a
 * threshold that you set are BLACK, and above that threshold, they
 * are WHITE. 
 *  image: the image to polarize
 *  returns: no return value
 */
void polarize(unsigned char image[][COLS]) { 
	printf("inside polarize\n");
	return;
}
/************************************************************************/
/* Function: sort_rows:
 * sort each row in increasing order of pixel, implment the bubble sort
 * algorithm to do this
 *
 *  image: the image to sort
 */
void sort_rows(unsigned char image[][COLS]) {
	printf("inside sort_rows\n");
	return;
}

/************************************************************************/
/* you can add more functions here */






/************************************************************************/
/* These are functions that you shouldn't modify.  They are for displaying
 * the image each time the user selects one of your image modification
 * options 
 */

/* Function: Tk_AppInit
 *   initialization funtion for gui display of picture
 *   DON'T MODIFY THIS FUNCTION
 */
int Tk_AppInit(Tcl_Interp *interp) {
        Tk_Window main;

        main = Tk_MainWindow(interp);
        if(Tcl_Init(interp) == TCL_ERROR) {
                return TCL_ERROR;
        }
        if(Tk_Init(interp) == TCL_ERROR) {
                return TCL_ERROR;
        }
        Tcl_EvalFile( interp, "display.tcl" );
        return TCL_OK;
}
/*
 *   DON'T MODIFY THIS FUNCTION
 *   returns -1 on error, 0 otherwise
 */
int InitGUI(Tcl_Interp **interp, char *argv[], 
		char *infile,
		unsigned char image[][COLS]) 
{

	Tcl_FindExecutable(argv[0]); 
	*interp = Tcl_CreateInterp();
  	if(gtiff_read2raster(infile, image)) {
		return(-1);
	} 
	if(gtiff_raster2pgmfile("altered.pgm", image)) {
		 perror("Error creating altered output file\n");
		 return(-1);
	}
	Tk_AppInit(*interp); 
	while (Tcl_DoOneEvent(TCL_ALL_EVENTS | TCL_DONT_WAIT)) { }
	return(0);
	
}
/*
 *   DON'T MODIFY THIS FUNCTION
 */
int Redisplay(unsigned char image[][COLS], Tcl_Interp *interp) { 
	if(gtiff_raster2pgmfile("altered.pgm", image)) { 
		perror("Error creating altered output file\n"); 
		return -1;
	} 
	Tcl_Eval(interp, "ReDraw"); 
	while (Tcl_DoOneEvent(TCL_ALL_EVENTS | TCL_DONT_WAIT)) { }
	return 0;
}
/************************************************************************/

